module Dag
  class Client
    module ClusterValidation
      def cluster_status
        return @cluster_status if @cluster_status

        if @cluster_info
          return @cluster_info['status']
        end

        return unless @cluster_name
        @cluster_info = @api.cluster_info(@cluster_name)
        @cluster_status = @cluster_info['status']

        @cluster_status
      end

      def valid_cluster_status?
        ['init', 'reserved', 'stopped', 'restarting', 'norm', 'failed', 'ptfailed'].include?(cluster_status)
      end

      def valid_cluster_info_list_status?(status = nil)
        status = cluster_status if status.nil?
        ['init', 'reserved', 'stopped', 'starting', 'restarting', 'norm', 'failed', 'ptfailed', 'error'].include?(status)
      end

      def cluster_norm?
        'norm' == cluster_status
      end

      def cluster_norm_or_ptfailed?
        ['norm', 'ptfailed'].include?(cluster_status)
      end

      def cluster_restart_status?
        ['norm', 'failed', 'ptfailed'].include?(cluster_status)
      end

      def validate_cluster
        raise Dag::Client::ClusterNotOpen.new("cluster not opened") if @cluster_name.blank?
        raise Dag::Client::StatusInvalid.new("cluster is not valid status: #{cluster_status}") unless valid_cluster_status?
      end

      VALID_WHERE_KEYS = [
        :status,
        :type,
        :cluster_name
      ]

      def validate_cluster_param_keys(params)
        params.keys.each do |k|
          unless VALID_WHERE_KEYS.include?(k.to_sym)
            raise Dag::Client::ParameterInvalid.new("Invalid where condition: #{k}")
          end
        end
      end
    end
  end
end
