module Dag
  class Client
    module Database
      include Dag::Client::ClusterValidation

      def databases
        validate_cluster

        Dag::DatabaseCollection.new(@api, @cluster_name, cluster_status: @cluster_status)
      end

      def create_database(db_name)
        validate_cluster

        @api.create_database(@cluster_name, db_name)
        Dag::Database.new(@api, @cluster_name, db_name)
      end

      def database(db_name)
        validate_cluster

        raise Dag::Client::ParameterInvalid.new("db_name is blank")if db_name.blank?

        unless db = databases.detect { |d| d.db_name == db_name }
          raise Dag::Client::DatabaseNotFound.new('Database not found')
        end

        db
      end

      def table(db_name, tbl_name)
        validate_cluster

        raise Dag::Client::ParameterInvalid.new("db_name is blank")if db_name.blank?
        raise Dag::Client::ParameterInvalid.new("tbl_name is blank")if tbl_name.blank?

        unless table_info = @api.table(@cluster_name, db_name, tbl_name)
          raise Dag::Client::TableNotFound.new('Table not found')
        end

        Dag::Table.new(@api, @cluster_name, db_name, params: table_info)
      end

      #
      # == parameters ==
      # * <tt>table</tt> - table name
      # * <tt>format</tt> - 'csv' or 'tsv' or 'json' or 'json_agent'
      # * <tt>schema/tt> - schema
      # * <tt>comment</tt> - comment
      def create_table(db_name, tbl_name, format: nil, schema: nil, comment: nil)
        validate_cluster

        params = {
          table: tbl_name,
          schema: schema,
          create_api: true
        }
        params.merge!({ format: format }) if format
        params.merge!({ comment: comment }) if comment

        @api.create_table(@cluster_name, db_name, params: params)
        table_info = @api.table(@cluster_name, db_name, tbl_name)
        Dag::Table.new(@api, @cluster_name, db_name, params: table_info)
      end

      def split_table(db_name, tbl_name, params)
        unless cluster_norm?
          raise Dag::Client::StatusInvalid.new("cluster status is invalid: #{cluster_status}")
        end

        split_info = @api.split_table(@cluster_name, db_name, tbl_name, params)
        job_id = split_info['queryId']

        query_info = @api.query_info(job_id)
        Dag::Job.new(@api, query_info)
      end
    end
  end
end
