module Dag
  class ObjectCollection < Model
    include Enumerable

    def initialize(api, bucket_name)
      super(api)
      @bucket = bucket_name
    end

    def [](object_name)
      object_named(object_name)
    end

    def where(prefix: nil, delimiter: nil)
      @prefix = prefix
      @delimiter = delimiter

      self
    end

    def each
      options = {}
      if @prefix
        options = options.merge(prefix: @prefix)
      end

      if @delimiter
        options = options.merge(delimiter: @delimiter)
      end

      marker = nil
      truncated = false
      begin
        if marker.present?
          options = options.merge(marker: marker)
        end
        objects_result = @api.objects(@bucket, options)
        objects = objects_result.full_objects

        truncated = objects_result.truncated?
        next_marker = objects_result.next_marker
        if next_marker.nil?
          marker = objects.last['Key'][0] if objects.present?
        else
          marker = next_marker
        end

        objects.each do |object|
          yield object_opts(object)
        end
      end while truncated
    end

    private

    def object_named(object_name)
      Dag::Object.new(@api, @bucket, object_name.to_s)
    end

    def object_opts(object_opts)
      Dag::Object.new(@api, @bucket, object_opts["Key"][0], object_opts)
    end
  end
end
