module Dag
  class Table < Model
    def initialize(api, cluster_name, db_name, params: {})
      super(api)
      @cluster_name = cluster_name
      @db_name = db_name
      load_table_info(params)
    end

    attr_reader :cluster_name, :db_name, :name, :format, :comment, :location, :created_at, :modified_at

    def update(schema: nil, format: nil, comment: nil)
      parameters = {
        table: @name,
        comment: comment || @comment,
        format: format || @table_info['format'],
        schema: schema || current_schema,
      }

      @api.create_table(@cluster_name, @db_name, params: parameters)
      info = @api.table(@cluster_name, @db_name, @name)
      load_table_info(info)

      self
    end

    def delete
      @api.delete_table(@cluster_name, @db_name, @name)
    end

    def current_schema
      @columns.map { |c| "#{c['name']} #{c['type']}" }.join(", ") if @columns.present?
    end

    alias :schema :current_schema

    def self.json_agent_partition_keys?(partition)
      (partition['name'] == 'date' && partition['type'] == 'string') ||
        (partition['name'] == 'hour' && partition['type'] == 'string')
    end

    private

    def load_table_info(params)
      @table_info = params
      @name = @table_info["tableName"]
      @format = @table_info["format"]
      @comment = @table_info["comment"]
      @location = @table_info["location"]
      @created_at = Time.at(@table_info["createTime"])
      @modified_at = Time.at(@table_info["modifiedTime"])
      @columns = params['columns']
    end
  end
end
