module Dag
  class TableCollection < Model
    include Enumerable
    include Dag::Client::ClusterValidation

    attr_reader :cluster_name

    def initialize(api, cluster_name, db_name)
      super(api)

      @cluster_name = cluster_name
      @db_name = db_name
    end

    def each
      marker = nil
      truncated = false
      begin
        table_info_list = @api.table_info_list(@cluster_name, @db_name, make_options(marker))
        table_info_list['tables'].each do |table_info|
          yield Dag::Table.new(@api, @cluster_name, @db_name, params: table_info)
        end
        truncated = table_info_list['isTruncated']
        marker = table_info_list['nextMarker']
      end while truncated
    end

    #
    # == parameters ==
    # * <tt>table</tt> - table name
    # * <tt>format</tt> - 'csv' or 'tsv' or 'json' or 'json_agent'
    # * <tt>schema/tt> - schema
    # * <tt>comment</tt> - comment
    def create(table: '', format: nil, schema: nil, comment: nil)
      params = {
        table: table,
        schema: schema,
        create_api: true
      }
      params.merge!({ format: format }) if format
      params.merge!({ comment: comment }) if comment

      @api.create_table(@cluster_name, @db_name, params: params)
      table_info = @api.table(@cluster_name, @db_name, table)
      Dag::Table.new(@api, @cluster_name, @db_name, params: table_info)
    end

    private

    def make_options(marker = nil)
      options = { max: 100 }

      if marker
        options = options.merge(marker: marker)
      end

      options
    end
  end
end
